#!/usr/bin/env python

"""
Pandoc filter to convert all level 2+ headers to paragraphs with
emphasized text.
"""

from pandocfilters import toJSONFilter, Emph, Para

def behead(key, value, format, meta):
  if key == 'Header' and value[0] == 1:
    return Para([Emph(value[2])])


if __name__ == "__main__":
  toJSONFilter(behead)
